# """siembra

# Usage:
#   siembra install <package> <output-directory>

# Options:
#   -h --help     Show this screen.
# """

# from docopt import docopt
# from pathlib import Path
# import yaml

# from siembra import *


# def main():
#     args = docopt(__doc__, version=SIEMBRA_VERSION)

#     if args['install']:
#         package = Path(args['<package>']).resolve()
#         output_directory = Path(args['<output-directory>']).resolve()

#         spec = yaml.load(open(package / 'siembra.yml'), Loader=yaml.Loader)

#         working_files = WorkingFiles(output=output_directory)
#         try:
#             installer = Siembra.installer(
#                 spec=spec,
#                 package=package,
#                 output=output_directory,
#                 working_files=working_files,
#             )

#             installer.run()
#         finally:
#             working_files.destroy()


# if __name__ == '__main__':
#     main()



"""siembra

Usage:
  siembra install <package> <output-directory>

Options:
  -h --help     Show this screen.
"""

from docopt import docopt
from pathlib import Path
import yaml

from siembra import (
    __version__,
    TempFiles,
    Siembra,
)


def main():
    args = docopt(__doc__, version=__version__)

    if args['install']:
        package = Path(args['<package>']).resolve()
        output_directory = Path(args['<output-directory>']).resolve()

        spec = yaml.load(open(package / 'siembra.yml'), Loader=yaml.Loader)

        temp_files = TempFiles(output=output_directory)
        try:
            installer = Siembra.installer(
                spec=spec,
                package=package,
                output=output_directory,
                temp_files=temp_files,
            )

            installer.run()
        finally:
            temp_files.destroy()


if __name__ == '__main__':
    main()



class Siembra:
    @staticmethod
    def get_module(version):
        from siembra import v0_1

        version_map = {
            'v0.1': v0_1,
        }

        module = version_map.get(version)
        return module

    @staticmethod
    def installer(spec, package, output, temp_files):
        version = spec.get('version')
        module = Siembra.get_module(version)

        if module:
            module.SiembraValidator(spec).validate()
            return module.SiembraInstaller(
                spec=spec,
                package=package,
                output=output,
                temp_files=temp_files,
            )
        else:
            raise Exception('FIXME')



import setuptools
import os
import re


with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()


def get_version():
    VERSIONFILE = os.path.join('siembra', '__init__.py')
    initfile_lines = open(VERSIONFILE, 'rt').readlines()
    VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
    for line in initfile_lines:
        mo = re.search(VSRE, line, re.M)
        if mo:
            return mo.group(1)
    raise RuntimeError('Unable to find version string in %s.' % (VERSIONFILE,))


setuptools.setup(
    name='siembra',
    version=get_version(),
    author='Juan Cabrera',
    author_email='juan@jjcp.dev',
    description='Description',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mulitadev/siembra',
    packages=[
        'siembra',
    ],
    install_requires=[
        'docopt==0.6.2',
        'PyYAML==5.3.1',
        'schema==0.7.2',
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>= 3.8',
    entry_points={
        'console_scripts': [
            'siembra=siembra.__main__:main',
        ],
    },
)



import shutil
from uuid import uuid4
from pathlib import Path


class TempFiles:
    def __init__(self, output):
        tmp_dir = (output / '__siembra_delete_me__').resolve()
        if tmp_dir.exists() and not tmp_dir.is_dir():
            tmp_dir.unlink()
        tmp_dir.mkdir(parents=True, exist_ok=True)

        self.tmp_dir = tmp_dir

    def destroy(self):
        shutil.rmtree(self.tmp_dir, ignore_errors=True)

    def create(self):
        max_retries = 3
        for try_count in range(max_retries):
            try:
                path = self.tmp_dir / str(uuid4())
                with open(path, 'x'):
                    pass
                return path
            except FileExistsError as e:
                if try_count != max_retries - 1:
                    continue
                raise e
